getGenePosition <- function(species="Mus musculus", type=c("protein_coding", "lincRNA", "miRNA")) {

    mart <- useMart("ensembl")
    tmp <- listDatasets(mart)
    dataset <- as.character(tmp[grep(species, tmp[,2]),1])
    mart <- useDataset(dataset, mart=mart)
    
    info <- getBM(attributes=c("ensembl_gene_id", "ensembl_transcript_id", "chromosome_name", "strand", "transcript_start", "transcript_end", "gene_biotype"), mart=mart)
    
    if (! is.null(type)) {
        info <- info[which(info$gene_biotype %in% type),]
    }
    
    info <- cbind(info, tss=findTSS(info$strand, info$transcript_start, info$transcript_end))
    
    geneID2tss <- split(info$tss, info$ensembl_gene_id)
    geneID2strand <- split(info$strand, info$ensembl_gene_id)
    geneID2strand <- lapply(geneID2strand, unique)
    geneID2strand <- unlist(geneID2strand)
    geneID2tss <- mapply(function(a,b) ifelse(b==1, min(a), max(a)), geneID2tss, geneID2strand[names(geneID2tss)])
    
    info <- data.frame(geneID=names(geneID2tss), chromosome=info[match(names(geneID2tss), info$ensembl_gene_id), "chromosome_name"], strand=info[match(names(geneID2tss), info$ensembl_gene_id), "strand"], tss=geneID2tss, stringsAsFactors=FALSE)
    
    info$chromosome <- sapply(info$chromosome, function(x) paste(c("chr", x), collapse=""))
    
    return(info)
}